######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ap.4gm/i_apcshe.4gs/detail.4gl  1.1  Delta: 2/13/91
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    m_popend record like stpopend.*,  # record like open items
    lookup_prep char(1), # Have the lookups been prepared?
    insert_prep char(1), # Has the insert statement been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=row, 2=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function lld_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    define
        goto_top  smallint,  # (boolean) Go to the top of the array?
        in_insert smallint   # (boolean) true if we're in 'insert row'

    # Trap fatal errors
    whenever error call error_handler

    # Set some initial values
    let goto_top = true
    # no detail lines, assume the tab key is pressed
    if rec1_cnt = 0
    then
      call fg_err(1) # no outstanding open items for this vendor
      return -1
    end if

    # This loop provides the ability to goto the top of the array.
    # Logic contained within the loop is not indented for readability.
    while goto_top

    let goto_top = false
    let in_insert = false
    let exit_level = 0
    let tab_pressed = false
    call set_count(rec1_cnt)
    input array p_pcashd without defaults from s_pcashd.*

      # The following section is for array type inputs
      before row
        if in_insert
        then
            # F1/F2 Scenario - fix parallel array if pre-informix 4.0
            if not is_4_0() and rec1_cnt > 0
              then call lld_a_delete() end if
            let in_insert = false  # Reset in_insert flag
        end if
        let scr_fld = ""           # Reset the current field
        let p_cur = arr_curr()     # Set the current array line
        let s_cur = scr_line()     # Set the current screen line
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_row()
        goto next_field
      after row
        call lld_showline()
        call lld_a_row()
        goto next_field
      before insert
        let in_insert = true
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_insert()
        goto next_field
      after insert
        let in_insert = false
        call lld_a_insert()
        goto next_field
      before delete
        call lld_b_delete()
        goto next_field
      after delete
        call lld_a_delete()
        goto next_field

      # All entry fields must have before and after field processing
      before field pay_method call lld_b_field("formonly.pay_method")
        goto next_field
      before field inv_no call lld_b_field("stpopend.inv_no")
        goto next_field
      before field due_date call lld_b_field("stpopend.due_date")
        goto next_field
      before field disc_date call lld_b_field("stpopend.disc_date")
        goto next_field
      before field disc_bal call lld_b_field("stpopend.disc_bal")
        goto next_field
      before field balance call lld_b_field("stpopend.balance")
        goto next_field
      before field disc_amt call lld_b_field("formonly.disc_amt")
        goto next_field
      before field paid call lld_b_field("formonly.paid")
        goto next_field

      after field pay_method call lld_a_field()
        goto next_field
      after field inv_no call lld_a_field()
        goto next_field
      after field due_date call lld_a_field()
        goto next_field
      after field disc_date call lld_a_field()
        goto next_field
      after field disc_bal call lld_a_field()
        goto next_field
      after field balance call lld_a_field()
        goto next_field
      after field disc_amt call lld_a_field()
        goto next_field
      after field paid call lld_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 2 then call lld_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after row' logic if necessary
        if exit_level >= 1 then call lld_a_row() end if
        if nxt_fld is not null then goto next_field end if

        # Don't run 'after input' logic if looping to top
        if goto_top then exit input end if

        # Run the 'after input' logic before exiting input
        call lld_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_local("cancel")
        call hot_key("lld_input")  # Map the key to the event
        call lld_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "pay_method" next field pay_method
          when nxt_fld = "inv_no" next field inv_no
          when nxt_fld = "due_date" next field due_date
          when nxt_fld = "disc_date" next field disc_date
          when nxt_fld = "disc_bal" next field disc_bal
          when nxt_fld = "balance" next field balance
          when nxt_fld = "disc_amt" next field disc_amt
          when nxt_fld = "paid" next field paid
          when nxt_fld = "exit input" goto end_input
          when nxt_fld = "goto top"
            call mld_arr_count()
            let goto_top = true
            goto end_input
        end case

    end input

    # This is the end of the 'goto top' loop.
    end while

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# lld_input()

######################################################################
function lld_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field and row level exit
    let exit_level = 2

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# lld_b_field()

######################################################################
function lld_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # Require row level exit
    let exit_level = 1

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "pay_method"
            if p_pcashd[p_cur].pay_method matches "[iI]"
            then
                let scratch =
                  " stpopend.doc_no = \"", q_pcashd[p_cur].inv_doc_no, "\""
                if zoom("apinvcz","(see scratch)") then end if
                let p_pcashd[p_cur].pay_method = null
                let nxt_fld = "pay_method"
                return
            end if
            if p_pcashd[p_cur].pay_method < "1" or
               p_pcashd[p_cur].pay_method > "4"
            then
                call fg_err(10) # Payment method must be between 1 and 4
                let p_pcashd[p_cur].pay_method = null
                let nxt_fld = "pay_method"
                return
            end if
            call pay_ap_line(p_cur,p_pcashd[p_cur].pay_method)
            let p_pcashd[p_cur].pay_method = null
            display p_pcashd[p_cur].* to s_pcashd[s_cur].*
            display by name p_pcashe.paid_tot, p_pcashe.disc_tot,
                    p_pcashe.chk_amt
        end case
    end if

end function
# lld_a_field()

######################################################################
function lld_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#

    if tab_pressed = false
      then call vend_unlock() end if
end function
# lld_a_input()

######################################################################
function lld_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "cancel"
        call vend_unlock()
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# lld_event()

######################################################################
function lld_b_row()
######################################################################
# This function is called before you enter a new row.
#

end function
# lld_b_row()

######################################################################
function lld_a_row()
######################################################################
# This function is called whenever you leave a row.
#
    # No more exit levels required
    let exit_level = 0

end function
# lld_a_row()

######################################################################
function lld_b_insert()
######################################################################
# This function is called before a new row is added to the array.
#
    define
        n smallint   # Generic counter

    # Expand (shift) the parallel array
    for n = (rec1_cnt - 1) to p_cur step -1
        let q_pcashd[n+1].* = q_pcashd[n].*
    end for

    # Blank out the current array element
    initialize q_pcashd[p_cur].* to null

    let nxt_fld = "goto top"
    return
end function
# lld_b_insert()

######################################################################
function lld_a_insert()
######################################################################
# This function is called after a new row has been added to the array.
#

end function
# lld_a_insert()

######################################################################
function lld_b_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and before the actual array elements have been shifted.
#
    let p_pcashe.disc_tot = p_pcashe.disc_tot -
                            p_pcashd[p_cur].disc_amt
    let p_pcashe.paid_tot = p_pcashe.paid_tot - p_pcashd[p_cur].paid
    let p_pcashe.chk_amt = p_pcashe.chk_amt -
        p_pcashd[p_cur].paid + p_pcashd[p_cur].disc_amt
    display by name p_pcashe.disc_tot,
                    p_pcashe.paid_tot, p_pcashe.chk_amt

end function
# lld_b_delete()

######################################################################
function lld_a_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and after the actual array elements have been shifted.
# (rec1_cnt contains the number of elements before the delete)
#
    define
        n smallint   # Generic counter

    # Compress (shift) the parallel array
    for n = (p_cur + 1) to rec1_cnt
        let q_pcashd[n-1].* = q_pcashd[n].*
    end for

    # Blank out the last (duplicate) data element
    initialize p_pcashd[rec1_cnt].* to null
    initialize q_pcashd[rec1_cnt].* to null

    # Perform array math
    call llh_math()
    call llh_display()

end function
# lld_a_delete()

######################################################################
function lld_setdata()
######################################################################
# This function is called to set the this_data variable
#
    case
      when scr_fld = "pay_method"
        call set_data(p_pcashd[p_cur].pay_method)
    end case
end function
# lld_setdata()

######################################################################
function lld_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "pay_method"
        display p_pcashd[p_cur].pay_method
          to s_pcashd[s_cur].pay_method attribute(reverse)
    end case
end function
# lld_high()

######################################################################
function lld_showline()
######################################################################
# This function displays the current p_* variables onto the screen line
#
    display p_pcashd[p_cur].* to s_pcashd[s_cur].*
end function
# lld_showline()

######################################################################
function lld_display()
######################################################################
# This function displays the first screenful of data
#
    for p_cur = 1 to scr1_max
        display p_pcashd[p_cur].* to s_pcashd[p_cur].*  attribute(red)
    end for
end function
# lld_display()

######################################################################
function lld_p_prep(n)
######################################################################
# This function creates the p_[]* record from the m_* record
#
    define
        n smallint      # Array element to process


    let q_pcashd[n].inv_doc_no = m_pcashd.inv_doc_no
    let q_pcashd[n].dist_acct = m_pcashd.dist_acct
    let q_pcashd[n].dist_department = m_pcashd.dist_department
    if m_pcashd.dist_amt is null
      then let m_pcashd.dist_amt = 0 end if
    if m_pcashd.disc_amt is null
      then let m_pcashd.disc_amt = 0 end if
    select * into m_popend.* from stpopend
        where stpopend.doc_no = m_pcashd.inv_doc_no
    if sqlca.sqlcode = notfound
    then
        let m_popend.disc_bal = 0
        let m_popend.balance = 0
    end if
    let q_pcashd[n].inv_doc_no = m_popend.doc_no
    let p_pcashd[n].disc_date = m_popend.disc_date
    let p_pcashd[n].inv_no = m_popend.inv_no
    let p_pcashd[n].due_date = m_popend.due_date

    if m_pcashd.disc_deb_cred = "CR"
    then
        let p_pcashd[n].disc_bal = m_popend.disc_bal -
                                   m_pcashd.disc_amt
    else
        let p_pcashd[n].disc_bal = m_popend.disc_bal +
                                   m_pcashd.disc_amt
    end if
    if m_pcashd.dist_deb_cred = "CR"
    then
        let p_pcashd[n].balance = m_popend.balance +
                                  m_pcashd.dist_amt
    else
        let p_pcashd[n].balance = m_popend.balance -
                                  m_pcashd.dist_amt
    end if
    let m_pcashd.dist_acct = m_popend.ap_acct_no
    let m_pcashd.dist_department = m_popend.ap_department
    let p_pcashd[n].pay_method = null
    let q_pcashd[n].dist_acct = m_pcashd.dist_acct
    let q_pcashd[n].dist_department = m_pcashd.dist_department
    let p_pcashd[n].paid = m_pcashd.dist_amt
    if m_pcashd.dist_deb_cred = "CR"
    then
        let p_pcashd[n].paid = m_pcashd.dist_amt * -1
    end if
    let p_pcashd[n].disc_amt = m_pcashd.disc_amt
    if m_pcashd.disc_deb_cred = "DB"
    then
        let p_pcashd[n].disc_amt = m_pcashd.disc_amt * -1
    end if
    let p_pcashe.disc_tot = p_pcashe.disc_tot + p_pcashd[n].disc_amt
    let p_pcashe.paid_tot = p_pcashe.paid_tot + p_pcashd[n].disc_amt
    if llh_lookup("pay_look",false) = false
    then  # p_pcashe.pay_to_code = "PAYTO"
      let p_pcashe.pay_to_name = p_pcashe.bus_name
    end if

end function
# lld_p_prep()

######################################################################
function lld_m_prep(n)
######################################################################
# This function creates the m_* record from the specified element
# of the p_.* array, and from elements of the header that tie the
# detail lines to the header
#
    define
        n smallint      # Array element to process

    # Items from the header...
    let m_pcashd.doc_no = m_pcashe.doc_no

    # Items from the p_ record array...
    let m_pcashd.inv_doc_no = q_pcashd[n].inv_doc_no
    let m_pcashd.dist_acct = q_pcashd[n].dist_acct
    let m_pcashd.dist_department = q_pcashd[n].dist_department
    let m_pcashd.inv_no = p_pcashd[n].inv_no
    let m_pcashd.due_date = p_pcashd[n].due_date
    if p_pcashd[n].paid is not null
    then
        let m_pcashd.dist_deb_cred = "DB"
        let m_pcashd.dist_amt = p_pcashd[n].paid
        if p_pcashd[n].paid < 0
        then
            let m_pcashd.dist_amt = p_pcashd[n].paid * -1
            let m_pcashd.dist_deb_cred = "CR"
        end if
    end if
    let m_pcashd.disc_acct = m_pcntrc.cd_disc_acct_no
    let m_pcashd.disc_amt = p_pcashd[n].disc_amt
    let m_pcashd.disc_department = m_pcashd.dist_department
    if m_pcashd.disc_amt < 0
    then
        let m_pcashd.disc_amt = m_pcashd.disc_amt * -1
        let m_pcashd.disc_deb_cred = "DB"
    else
        let m_pcashd.disc_deb_cred = "CR"
    end if

end function
# lld_m_prep()

######################################################################
function lld_delete()
######################################################################
# This function deletes the rows in the detail table
# that match the where clause
#
    delete from stpcashd
      where stpcashd.doc_no = m_pcashe.doc_no
end function
# lld_delete()

