########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ap.4gm/i_apcshe.4gs/fg_funcs.4gl  1.3  Delta: 2/13/91
########################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    insert_prep char(1), # Has the insert statement been prepared?
    select_prep char(1)  # Has the select statement been prepared?

#######################################################################
function calc_on_acct()
# returning the calculated on account amount
#######################################################################
# this function calculates the actual on account amount by total
# all similar cash receipts's applied on account amount
#
    define
        db_oa_amt like stpvendr.on_acct_amt,
        cr_oa_amt like stpvendr.on_acct_amt

    select on_acct_amt into
        p_pcashe.on_acct_amt from stpvendr
        where vend_code = p_pcashe.vend_code

    if sqlca.sqlcode = notfound
    then
        let p_pcashe.on_acct_amt = 0
    end if

    select sum(oa_amt) into db_oa_amt from stpcashe where
        stpcashe.vend_code = p_pcashe.vend_code and
        stpcashe.oa_deb_cred = "DB"

    select sum(oa_amt) into cr_oa_amt from stpcashe where
        stpcashe.vend_code = p_pcashe.vend_code and
        stpcashe.oa_deb_cred = "CR"

    if db_oa_amt is null then let db_oa_amt = 0 end if
    if cr_oa_amt is null then let cr_oa_amt = 0 end if

    let p_pcashe.on_acct_amt = p_pcashe.on_acct_amt + cr_oa_amt - db_oa_amt

end function
# calc_on_acct

#######################################################################
function automatic_pay()
#######################################################################
# pays the on account balance first, then scans through the invoice
# lines and pays those that have a balance left in them.  any money
# left over is applied to "on account".
#
    define
        n integer # generic counter

    let p_pcashe.paid_tot = 0
    let p_pcashe.disc_tot = 0
    let p_pcashe.oa_amt = p_pcashe.on_acct_amt
    let p_pcashe.chk_amt = p_pcashe.oa_amt
    for n = 1 to rec1_cnt
        if p_pcashd[n].balance is null
        then
            continue for
        end if
        let p_pcashd[n].balance = p_pcashd[n].balance + p_pcashd[n].paid
        let p_pcashd[n].paid = 0
        let p_pcashd[n].disc_bal = p_pcashd[n].disc_bal +
                                   p_pcashd[n].disc_amt
        let p_pcashd[n].disc_amt = 0
        call pay_ap_line(n,1)
    end for
    if p_pcashe.chk_amt < 0
    then
        let p_pcashe.oa_amt = p_pcashe.oa_amt - p_pcashe.chk_amt
        let p_pcashe.chk_amt = 0
    end if
    call llh_display()
    call lld_display()
end function
# automatic_pay

#######################################################################
function new_vend()
#######################################################################
# get a new vendor's information and open items
#
    select ap_acct_dflt, ap_department_dflt, on_acct_amt into
        oa_acct, oa_department,
        p_pcashe.on_acct_amt from stpvendr
        where vend_code = p_pcashe.vend_code
    if not vend_lock(p_pcashe.vend_code)
    then
        call fg_err(12) # Cannot lock vendor record.
        return false
    end if
    call init_vend_header()
    call llh_display()
    call init_vend_detail(0)
    if sqlca.sqlcode = -1
    then
        let sqlca.sqlcode = 0
        call fg_err(9) # Too many invoices in stpopend to read
        let int_flag = -1
        return false
    end if
    call lld_display()
    return true
end function
# new_vend()

#######################################################################
function init_vend_header()
#######################################################################
# sets up all the header amounts for a new or changed vendor
    define
        p_oa_amt like stpcashe.oa_amt

    let m_pcashe.vend_code = p_pcashe.vend_code

    call calc_on_acct()

    if p_pcashe.chk_amt is null
    then
        let p_pcashe.chk_amt = 0
    end if
    if p_pcashe.on_acct_amt is null
    then
        let p_pcashe.on_acct_amt = 0
    end if
    let p_pcashe.disc_tot = 0
    let p_pcashe.paid_tot = 0
    let p_pcashe.oa_amt = 0
    if oa_acct is null
    then
        let oa_acct = m_pcntrc.ap_acct_no
    end if
    if oa_department is null
    then
        let oa_department = "000"
    end if
end function
# init_vend_header

#######################################################################
function init_vend_detail(n)
#######################################################################
#   - this function reads all the vendor's non-zero invoices into
#     the p_pcashd array
#   - it uses a cursor, but doesn't lock any rows.
#   - it reads into the m_* record, and moves each item to the p_* array
#
    define
        n           smallint,   # working number
        scratch_date date
    if n = 0
    then
        call mld_clear()
    end if

    let m_pcashd.dist_amt = 0
    let m_pcashd.disc_amt = 0

    declare e_cursor cursor for
        select doc_no, due_date from stpopend where
            stpopend.vend_code = p_pcashe.vend_code and
            stpopend.pay_to_code = p_pcashe.pay_to_code and
            stpopend.cash_acct_no = p_pcashe.cash_acct and
            stpopend.cash_department = p_pcashe.cash_department and
            stpopend.balance != 0 and
            stpopend.doc_no not in 
              (select inv_doc_no from stpcashe, stpcashd where
                 stpcashe.vend_code = p_pcashe.vend_code and 
                 stpcashe.pay_to_code = p_pcashe.pay_to_code and 
                 stpcashe.cash_acct = p_pcashe.cash_acct and
                 stpcashe.cash_department = p_pcashe.cash_department and
                 stpcashe.doc_no = stpcashd.doc_no)
            order by stpopend.due_date, stpopend.doc_no

    foreach e_cursor into m_pcashd.inv_doc_no, scratch_date
        let n = n + 1
        if n > rec1_max
        then
            let rec1_cnt = 0
            let sqlca.sqlcode = -1
            exit foreach
        end if
        call lld_p_prep(n)            # move data from m_* to p_[].*
    end foreach
    let rec1_cnt = n
end function
# init_vend_detail()

#######################################################################
function pay_ap_line(n,pay_type)
#######################################################################
# pays an invoice line based on the pay type as follows:
#   1) auto pay.  take discount if within date.  pay all
#   2) take discount.  like 1 except discount always taken
#   3) no discount.  like 1 except discount never taken
#   4) manual.  ask discount and amount.
#
    define
        n smallint,
        pay_type char(1),
        old_pay_amt like stpcashd.dist_amt,
        pay_amt like stpcashd.dist_amt,
        old_discount like stpcashd.disc_amt,
        discount like stpcashd.disc_amt

    if p_pcashd[n].balance is null or pay_type is null or pay_type = " "
    then
        return
    end if
    let old_pay_amt = p_pcashd[n].paid - p_pcashd[n].disc_amt
    let p_pcashd[n].balance = p_pcashd[n].balance + p_pcashd[n].paid
    let p_pcashd[n].paid = 0
    let old_discount = p_pcashd[n].disc_amt
    let p_pcashd[n].disc_bal = p_pcashd[n].disc_bal +
                               p_pcashd[n].disc_amt
    let p_pcashd[n].disc_amt = 0
    let pay_amt = p_pcashd[n].balance
    let discount = 0
    let p_pcashe.chk_amt = p_pcashe.chk_amt - old_pay_amt
    let p_pcashe.disc_tot = p_pcashe.disc_tot - old_discount
    let p_pcashe.paid_tot = p_pcashe.paid_tot -
                            old_pay_amt - old_discount

    if (p_pcashd[n].disc_date >= p_pcashe.chk_date and
        pay_type = "1") or
       (p_pcashe.take_dscnt = "Y" and pay_type = "1") or
        pay_type = "2"
    then
        let discount = p_pcashd[n].disc_bal
        let pay_amt = pay_amt - discount
    end if

    if pay_type = "4"
    then
        prompt "Enter the pay amount: " for pay_amt
        if int_flag != 0
        then
            let int_flag = 0
            return
        end if
        prompt "Enter the discount amount: " for discount
        if int_flag != 0
        then
            let int_flag = 0
            return
        end if
        if pay_amt is null
        then
            let pay_amt = 0
        end if
        if discount is null
        then
            let discount = 0
        end if
    end if
    let p_pcashe.chk_amt = p_pcashe.chk_amt + pay_amt
    let p_pcashe.disc_tot = p_pcashe.disc_tot + discount
    let p_pcashe.paid_tot = p_pcashe.paid_tot + pay_amt + discount
    let p_pcashd[n].paid = p_pcashd[n].paid + pay_amt + discount
    let p_pcashd[n].disc_amt = p_pcashd[n].disc_amt + discount
    let p_pcashd[n].balance = p_pcashd[n].balance - pay_amt - discount
    let p_pcashd[n].disc_bal = p_pcashd[n].disc_bal - discount
end function
# pay_ap_line(n,pay_type)

##################################################################
function ok_update()
# returning true or false based upon ok to update
##################################################################
#
    if m_pcashe.ap_type = "Y"
    then
        if not vend_lock(m_pcashe.vend_code)
        then
             call fg_err(12) # "Cannot lock vendor row."
             return false
        end if
        call please_wait()
        call init_vend_detail(rec1_cnt)
        call lld_display()
    end if
    return true
end function
# ok_update
######################################################################
function llh_lookup(tbl_name,mustfind)
# returning false if notfound
######################################################################
# This is the lookup function.  It is passed a table to lookup, and
# based on that, it looks up the key and fills any variables for
# that table.  The lookup function will produce the error if it is
# called with mustfind=true, and a notfound situation exists.
#
    define
        tbl_name char(18),  # Lookup table name
        stat integer,       # Value of sqlca.sqlcode after lookup
        mustfind smallint   # (boolean) produce an error if notfound?
    # Prepare cursors upon first call to this function
    if lookup_prep is null
    then
        let lookup_prep = "Y"
        let scratch = "select bus_name, take_dscnt ",
          "from stpvendr where ",
          " stpvendr.vend_code = ?"
        prepare str_vend_lk from scratch
        declare cur_vend_lk cursor for str_vend_lk
        let scratch = "select pay_to_name, take_dscnt ",
          "from stppytor where ",
          " stppytor.vend_code = ? and",
          " stppytor.pay_to_code = ?"
        prepare str_pay_look from scratch
        declare cur_pay_look cursor for str_pay_look
        let scratch = "select rowid ",
          "from stxinfor where ",
          " stxinfor.src_type = \"D\" and",
          " stxinfor.src_key = ?"
        prepare str_stxinfor from scratch
        declare cur_stxinfor cursor for str_stxinfor
    end if
    # Perform the lookup
    case
      when tbl_name = "vend_lk"
        open cur_vend_lk using p_pcashe.vend_code
        fetch cur_vend_lk into p_pcashe.bus_name, q_pcashe.take_dscnt
        let stat = sqlca.sqlcode
        close cur_vend_lk
      when tbl_name = "pay_look"
        open cur_pay_look using p_pcashe.vend_code,
          p_pcashe.pay_to_code
        fetch cur_pay_look into p_pcashe.pay_to_name,
          p_pcashe.take_dscnt
        let stat = sqlca.sqlcode
        close cur_pay_look
      when tbl_name = "stxinfor"
        open cur_stxinfor using p_pcashe.cash_department
        fetch cur_stxinfor into scratch
        let stat = sqlca.sqlcode
        close cur_stxinfor
    end case
    # No mustfind if this_data is empty
    if length(this_data) = 0 then let mustfind = false end if
    # Send notfound message to user (if requested)
    let sqlca.sqlcode = stat
    if mustfind and stat = notfound
      then call scr_error("lookup=notfound",tbl_name)
    end if
    let sqlca.sqlcode = 0
    # Return false if lookup failed
    if stat = notfound
    then
        case
          when tbl_name = "vend_lk"
            initialize p_pcashe.bus_name to null
            initialize p_pcashe.on_acct_amt to null
            initialize q_pcashe.take_dscnt to null
          when tbl_name = "pay_look"
            initialize p_pcashe.pay_to_name to null
            initialize p_pcashe.take_dscnt to null
          when tbl_name = "stxinfor"
            initialize scratch to null
        end case
        return false
    else
        return true
    end if
end function
# llh_lookup()
######################################################################
function lld_read()
######################################################################
# This function reads in the array elements from disk.
# It first reads the data into the m_* records, then calls
# p_prep for each array element.
#
    define
        row_id integer  # Rowid of the fetched element
    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select rowid, * from stpcashd ",
          "where stpcashd.doc_no = ? order by due_date, inv_doc_no"
        prepare read_specs from scratch
        declare d_cursor cursor for read_specs
    end if
    # Read in the detail lines
    let p_cur = 0
    open d_cursor using m_pcashe.doc_no
    foreach d_cursor into row_id, m_pcashd.*
        let p_cur = p_cur + 1
        if p_cur > rec1_max
        then
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(p_cur)
        let q_pcashd[p_cur].row_id = row_id
    end foreach
    close d_cursor
    let rec1_cnt = p_cur
    call set_count(rec1_cnt)
end function
# lld_read()
######################################################################
function lld_add()
######################################################################
# This function inserts data into the detail table.
#
    # Declare an insert cursor for maximum efficiency
    if insert_prep is null
    then
        let insert_prep = "Y"
        declare d_insert cursor for
          insert into stpcashd values(m_pcashd.*)
    end if
    # Insert all array elements
    open d_insert
    for p_cur = 1 to rec1_cnt
        call lld_m_prep(p_cur)
        if m_pcashd.dist_amt != 0 or
           m_pcashd.disc_amt != 0
        then
            put d_insert
        end if
    end for
    close d_insert
end function
# lld_add()
