######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    amount like stxckrgd.amount,
    p_doc_date like stxtranr.doc_date,
    p_doc_desc like stxtranr.doc_desc,
    posted_to_gl like stgtranr.status,
    lookup_prep char(1), # Have the lookups been prepared?
    insert_prep char(1), # Has the insert statement been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=row, 2=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function lld_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    define
        goto_top  smallint,  # (boolean) Go to the top of the array?
        in_insert smallint   # (boolean) true if we're in 'insert row'

    # Trap fatal errors
    whenever error call error_handler

    # Set some initial values
    let goto_top = true
    if rec1_cnt < 1 then return -1 end if

    # This loop provides the ability to goto the top of the array.
    # Logic contained within the loop is not indented for readability.
    while goto_top

    let goto_top = false
    let in_insert = false
    let exit_level = 0
    let tab_pressed = false
    call set_count(rec1_cnt)
    input array p_xckrgd without defaults from s_ckrgd.*

      # The following section is for array type inputs
      before row
        if in_insert
        then
            # f1/f2 scenario - fix parallel array if pre-informix 4.0
            if not is_4_0() and rec1_cnt > 0
            then
                call lld_a_delete()
            end if
            let in_insert = false  # Reset in_insert flag
        end if
        let scr_fld = ""           # Reset the current field
        let p_cur = arr_curr()     # Set the current array line
        let s_cur = scr_line()     # Set the current screen line
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_row()
        goto next_field
      after row
        call lld_showline()
        call lld_a_row()
        goto next_field
      before insert
        let in_insert = true
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_insert()
        goto next_field
      after insert
        let in_insert = false
        call lld_a_insert()
        goto next_field
      before delete
        call lld_b_delete()
        goto next_field
      after delete
        call lld_a_delete()
        goto next_field

      # All entry fields must have before and after field processing
      before field reconciled call lld_b_field("stxckrgd.reconciled")
        goto next_field

      after field reconciled call lld_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 2 then call lld_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after row' logic if necessary
        if exit_level >= 1 then call lld_a_row() end if
        if nxt_fld is not null then goto next_field end if

        # Don't run 'after input' logic if looping to top
        if goto_top then exit input end if

        # Run the 'after input' logic before exiting input
        call lld_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("lld_input")  # Map the key to the event
        call lld_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "reconciled" next field reconciled
          when nxt_fld = "exit input" goto end_input
          when nxt_fld = "goto top"
            call mld_arr_count()
            let goto_top = true
            goto end_input
        end case

    end input

    # This is the end of the 'goto top' loop.
    end while

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# lld_input()

######################################################################
function lld_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field and row level exit
    let exit_level = 2

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# lld_b_field()

######################################################################
function lld_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # Require row level exit
    let exit_level = 1

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "reconciled"
            if p_xckrgd[p_cur].reconciled != "X" and
               p_xckrgd[p_cur].reconciled is not null
            then
                let p_xckrgd[p_cur].reconciled = null
                call lib_error("lib_scr", "lookup", 1, "")
                let nxt_fld = "reconciled"
                return
            end if

            if p_xckrgd[p_cur].reconciled = "X"
            then
                # changed from no to yes
                if p_db_cr = "DB"
                then
                    let amount = p_xckrgd[p_cur].amount * -1
                else
                    let amount = p_xckrgd[p_cur].amount
                end if
            else
                # changed from yes to no
                if p_db_cr = "DB"
                then
                    let amount = p_xckrgd[p_cur].amount
                else
                    let amount = p_xckrgd[p_cur].amount * -1
                end if
                let p_xckrgd[p_cur].reconciled = null
            end if
            let p_xckrgr.not_reconciled = p_xckrgr.not_reconciled + amount
            let p_xckrgr.stmt_balance = p_xckrgr.acct_balance -
                                        p_xckrgr.not_reconciled +
                                        p_xckrgr.adjustments
            call llh_display()
        end case
    end if

end function
# lld_a_field()

######################################################################
function lld_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#

end function
# lld_a_input()

######################################################################
function lld_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# lld_event()

######################################################################
function lld_b_row()
######################################################################
# This function is called before you enter a new row.
#

end function
# lld_b_row()

######################################################################
function lld_a_row()
######################################################################
# This function is called whenever you leave a row.
#
    # No more exit levels required
    let exit_level = 0

end function
# lld_a_row()

######################################################################
function lld_b_insert()
######################################################################
# This function is called before a new row is added to the array.
#
    define
        n smallint   # Generic counter

    # Expand (shift) the parallel array
    for n = (rec1_cnt - 1) to p_cur step -1
        let q_xckrgd[n+1].* = q_xckrgd[n].*
    end for

    # Blank out the current array element
    initialize q_xckrgd[p_cur].* to null

    let nxt_fld = "goto top"
    return
end function
# lld_b_insert()

######################################################################
function lld_a_insert()
######################################################################
# This function is called after a new row has been added to the array.
#

end function
# lld_a_insert()

######################################################################
function lld_b_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and before the actual array elements have been shifted.
#

end function
# lld_b_delete()

######################################################################
function lld_a_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and after the actual array elements have been shifted.
# (rec1_cnt contains the number of elements before the delete)
#
    define
        n smallint   # Generic counter

    # Compress (shift) the parallel array
    for n = (p_cur + 1) to rec1_cnt
        let q_xckrgd[n-1].* = q_xckrgd[n].*
    end for

    # Blank out the last (duplicate) data element
    initialize p_xckrgd[rec1_cnt].* to null
    initialize q_xckrgd[rec1_cnt].* to null

    # Perform array math
    call llh_math()
    call llh_display()

end function
# lld_a_delete()

######################################################################
function lld_setdata()
######################################################################
# This function is called to set the this_data variable
#
    case
      when scr_fld = "reconciled"
        call set_data(p_xckrgd[p_cur].reconciled)
    end case
end function
# lld_setdata()

######################################################################
function lld_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "reconciled"
        display p_xckrgd[p_cur].reconciled
          to s_ckrgd[s_cur].reconciled attribute(reverse)
    end case
end function
# lld_high()

######################################################################
function lld_showline()
######################################################################
# This function displays the current p_* variables onto the screen line
#
    display p_xckrgd[p_cur].* to s_ckrgd[s_cur].*
end function
# lld_showline()

######################################################################
function lld_display()
######################################################################
# This function displays the first screenful of data
#
    for p_cur = 1 to scr1_max
        display p_xckrgd[p_cur].* to s_ckrgd[p_cur].*  attribute(red)
    end for
end function
# lld_display()

######################################################################
function lld_p_prep(n)
######################################################################
# This function creates the p_[]* record from the m_* record
#
    define
        n smallint      # Array element to process

    let p_xckrgd[n].reconciled = m_xckrgd.reconciled
    let p_xckrgd[n].orig_journal = m_xckrgd.orig_journal
    let p_xckrgd[n].inv_chk_no = m_xckrgd.inv_chk_no
    let p_xckrgd[n].amount = m_xckrgd.amount

    let q_xckrgd[n].doc_no = m_xckrgd.doc_no
    let q_xckrgd[n].acct_no = m_xckrgd.acct_no
    let q_xckrgd[n].department = m_xckrgd.department
    let q_xckrgd[n].debit_credit = m_xckrgd.debit_credit
    let q_xckrgd[n].chk_voided = m_xckrgd.chk_voided
    call scr_d_rec_prep(n)

end function
# lld_p_prep()

######################################################################
function lld_m_prep(n)
######################################################################
# This function creates the m_* record from the specified element
# of the p_.* array, and from elements of the header that tie the
# detail lines to the header
#
    define
        n smallint      # Array element to process

    # Items from the header...
    let m_xckrgd.acct_no = m_xckrgr.acct_no
    let m_xckrgd.department = m_xckrgr.department

    # Items from the p_ record array...
    let m_xckrgd.reconciled = p_xckrgd[n].reconciled
    let m_xckrgd.orig_journal = p_xckrgd[n].orig_journal
    let m_xckrgd.inv_chk_no = p_xckrgd[n].inv_chk_no
    let m_xckrgd.amount = p_xckrgd[n].amount
    let m_xckrgd.doc_no = q_xckrgd[n].doc_no
    let m_xckrgd.acct_no = q_xckrgd[n].acct_no
    let m_xckrgd.department = q_xckrgd[n].department
    let m_xckrgd.debit_credit = q_xckrgd[n].debit_credit
    let m_xckrgd.chk_voided = q_xckrgd[n].chk_voided
    if m_xckrgd.reconciled is null
    then
        let m_xckrgd.reconciled = "N"
    else
        let m_xckrgd.reconciled = "Y"
    end if
    let m_xckrgd.amount = absolute(m_xckrgd.amount)

end function
# lld_m_prep()

######################################################################
function lld_delete()
######################################################################
# This function deletes the rows in the detail table
# that match the where clause
#
    delete from stxckrgd
      where stxckrgd.acct_no = m_xckrgr.acct_no
        and stxckrgd.department = m_xckrgr.department
end function
# lld_delete()

######################################################################
function lld_add()
######################################################################
# This function inserts data into the detail table.
#
    # Declare an insert cursor for maximum efficiency
    if insert_prep is null
    then
        let insert_prep = "Y"
        declare d_insert cursor for
          insert into stxckrgd values(m_xckrgd.*)
    end if

    # Insert all array elements
    open d_insert
    for p_cur = 1 to rec1_cnt
        call lld_m_prep(p_cur)
        put d_insert
    end for
    close d_insert

end function
# lld_add()


######################################################################
function lld_read()
######################################################################
# This function reads in the array elements from disk.
# It first reads the data into the m_* records, then calls
# p_prep for each array element.
#
    define
        row_id integer  # Rowid of the fetched element

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch =
            "select stxckrgd.rowid, stxckrgd.*, stxtranr.doc_date, ",
            "stxtranr.doc_desc, stgtranr.status ",
            "from stxckrgd, stxtranr, stgtranr ",
            "where stxckrgd.orig_journal = stxtranr.orig_journal ",
            "  and stxckrgd.doc_no = stxtranr.doc_no ",
            "  and stxckrgd.orig_journal = stgtranr.orig_journal ",
            "  and stxckrgd.doc_no = stgtranr.doc_no ",
            "  and stxckrgd.acct_no = ?",
            "  and stxckrgd.department = ?",
            "  and stxckrgd.reconciled = \"N\" ",
            "order by stxtranr.doc_date, stxckrgd.orig_journal, ",
#c->        " stxckrgd.doc_no "
            " stxckrgd.inv_chk_no "
        prepare read_specs from scratch
        declare d_cursor cursor for read_specs
    end if

    # Read in the detail lines
    let p_cur = 0
    open d_cursor using m_xckrgr.acct_no, m_xckrgr.department
    foreach d_cursor into row_id, m_xckrgd.*,
                          p_doc_date, p_doc_desc, posted_to_gl
        let p_cur = p_cur + 1
        if p_cur > rec1_max
        then
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(p_cur)
        let q_xckrgd[p_cur].row_id = row_id
    end foreach
    close d_cursor
    let rec1_cnt = p_cur
    call set_count(rec1_cnt)
end function
# lld_read()

#######################################################################
function scr_d_rec_prep(n)
#######################################################################
#
    define
        n smallint     # array element

    let p_xckrgd[n].reconciled   = null

    let p_xckrgd[n].doc_date   = p_doc_date
    let p_xckrgd[n].doc_desc   = p_doc_desc

    # line db and cr amounts,  cash accounts are usually debit accounts
    if p_db_cr = "DB"
    then
      if m_xckrgd.debit_credit = "D"
      then
        let p_xckrgd[n].amount = m_xckrgd.amount
      else
        let p_xckrgd[n].amount = m_xckrgd.amount * -1
      end if
    else
      if m_xckrgd.debit_credit = "C"
      then
        let p_xckrgd[n].amount = m_xckrgd.amount
      else
        let p_xckrgd[n].amount = m_xckrgd.amount * -1
      end if
    end if

    # update the balances in the header...

    # if not posted yet, then update acct_balance
    if posted_to_gl = "N"
    then
        let p_xckrgr.acct_balance = p_xckrgr.acct_balance +
                                    p_xckrgd[n].amount
        let p_xckrgr.stmt_balance = p_xckrgr.stmt_balance +
                                    p_xckrgd[n].amount
    end if
    # reduce the statement balance and put it
    # into the "not reconciled" column
    let p_xckrgr.stmt_balance = p_xckrgr.stmt_balance -
                                p_xckrgd[n].amount
    let p_xckrgr.not_reconciled = p_xckrgr.not_reconciled +
                                  p_xckrgd[n].amount

end function
# scr_d_rec_prep(n)
