######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    # Initialize some variables
    let exit_level = 0
    let tab_pressed = false


    input p_xckrgr.* without defaults from s_ckrgr.*

      # All entry fields must have before and after field processing
      before field adjustments call llh_b_field("formonly.adjustments")
        goto next_field

      after field adjustments call llh_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 1 then call llh_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after input' logic before exiting input
        call llh_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("llh_input")  # Map the key to the event
        call llh_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "adjustments" next field adjustments
          when nxt_fld = "exit input" goto end_input
        end case
    end input

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# llh_input()

######################################################################
function llh_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field level exit
    let exit_level = 1

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# llh_b_field()

######################################################################
function llh_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # No more exit levels required
    let exit_level = 0

    # After field logic
    case
      when scr_fld = "adjustments"
        if p_xckrgr.adjustments is null
          then let p_xckrgr.adjustments = 0 end if
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "adjustments"
            let p_xckrgr.adjustments = round("a",p_xckrgr.adjustments)
            let p_xckrgr.stmt_balance = p_xckrgr.acct_balance -
                                        p_xckrgr.not_reconciled +
                                        p_xckrgr.adjustments
        end case
    end if

end function
# llh_a_field()

######################################################################
function llh_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#

end function
# llh_a_input()

######################################################################
function llh_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# llh_event()

######################################################################
function llh_setdata()
######################################################################
# This function is called to set the this_data global variable.
#
    case
      when scr_fld = "adjustments"
        call set_data(p_xckrgr.adjustments)
    end case
end function
# llh_setdata()

######################################################################
function llh_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "adjustments"
        display by name p_xckrgr.adjustments attribute(reverse)
    end case
end function
# llh_high()

######################################################################
function llh_display()
######################################################################
# This function displays all p_* variables onto the screen
#
    display p_xckrgr.* to s_ckrgr.*  attribute(red)
end function
# llh_display()

######################################################################
function llh_p_prep()
######################################################################
# This function creates the p_* record from the m_* record
#
    let p_xckrgr.acct_no = m_xckrgr.acct_no
    let p_xckrgr.department = m_xckrgr.department

    # Perform lookups
    if llh_lookup("stxchrtr",false) then end if

    call scr_h_rec_prep()

end function
# llh_p_prep()

######################################################################
function llh_m_prep()
######################################################################
# This function creates the m_* record from the p_* record
#
    let m_xckrgr.acct_no = p_xckrgr.acct_no
    let m_xckrgr.department = p_xckrgr.department
end function
# llh_m_prep()

######################################################################
function llh_read(ring_rowid)
######################################################################
# This function loads the header p_* record elements.
# It first loads the m_* record from disk, then it
# calls the p_prep routine to fill the p_* record elements.
#
    define
        ring_rowid  integer     # rowid of record to read

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select * from stxckrgr where rowid = ?"
        prepare read_specs from scratch
        declare h_cursor cursor for read_specs
    end if

    # Read the header record
    open h_cursor using ring_rowid
    fetch h_cursor into m_xckrgr.*
    if sqlca.sqlcode != 0 then return end if
    let q_xckrgr.row_id = ring_rowid
    close h_cursor
    call llh_p_prep()
end function
# llh_read()

######################################################################
function llh_delete(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stxckrgr
        where rowid = ring_rowid
end function
# llh_delete()

######################################################################
function llh_add()
######################################################################
# This function inserts data into the header table.
#
    define
        new_rowid integer  # Rowid after insert
    # Insert the data
    insert into stxckrgr values(m_xckrgr.*)
    let new_rowid = sqlca.sqlerrd[6]

    # Reset rowid
    let sqlca.sqlerrd[6] = new_rowid

end function
# llh_add()

######################################################################
function llh_update(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stxckrgr set stxckrgr.* = m_xckrgr.*
      where rowid = ring_rowid
end function
# llh_update()

######################################################################
function llh_lookup(tbl_name,mustfind)
# returning false if notfound
######################################################################
# This is the lookup function.  It is passed a table to lookup, and
# based on that, it looks up the key and fills any variables for
# that table.  The lookup function will produce the error if it is
# called with mustfind=true, and a notfound situation exists.
#
    define
        tbl_name char(18),  # Lookup table name
        stat integer,       # Value of sqlca.sqlcode after lookup
        mustfind smallint   # (boolean) produce an error if notfound?

    # Prepare cursors upon first call to this function
    if lookup_prep is null
    then
        let lookup_prep = "Y"
        let scratch = "select acct_desc ",
          "from stxchrtr where ",
          " stxchrtr.acct_no = ?"
        prepare str_stxchrtr from scratch
        declare cur_stxchrtr cursor for str_stxchrtr
    end if

    # Perform the lookup
    case
      when tbl_name = "stxchrtr"
        open cur_stxchrtr using p_xckrgr.acct_no
        fetch cur_stxchrtr into p_xckrgr.acct_desc
        let stat = sqlca.sqlcode
        close cur_stxchrtr
    end case

    # No mustfind if this_data is empty
    if length(this_data) = 0 then let mustfind = false end if

    # Send notfound message to user (if requested)
    let sqlca.sqlcode = stat
    if mustfind and stat = notfound
      then call scr_error("lookup=notfound",tbl_name)
    end if
    let sqlca.sqlcode = 0

    # Return false if lookup failed and empty into fields
    if stat = notfound
    then
        case
          when tbl_name = "stxchrtr"
            initialize p_xckrgr.acct_desc to null
        end case
        return false
    else
        return true
    end if
end function
# llh_lookup()


#######################################################################
function scr_h_rec_prep()
#######################################################################
#
    define
        cur_period like stxperdr.period,
        cur_year like stxperdr.period_year

    # don't continue if a blank screen
    if m_xckrgr.acct_no is null then return end if

    if p_xckrgr.adjustments is null
      then let p_xckrgr.adjustments = 0 end if

    # account description
    select incr_with_crdt into p_db_cr
        from stxchrtr where acct_no = m_xckrgr.acct_no

    if p_db_cr = "N"
    then
        let p_db_cr = "DB"
    else
        let p_db_cr = "CR"
    end if

    # account balance (null if g/l isn't installed)
    whenever error continue
    select curr_period, curr_year into cur_period, cur_year
      from stgcntrc
    if sqlca.sqlcode = 0
    then
        select balance into p_xckrgr.acct_balance from stxchrtd
          where acct_no = m_xckrgr.acct_no
            and department = m_xckrgr.department
            and period_month = cur_period
            and period_year = cur_year
        if sqlca.sqlcode = 0
        then
            let p_xckrgr.not_reconciled = 0
            let p_xckrgr.adjustments = 0
            let p_xckrgr.stmt_balance = p_xckrgr.acct_balance
        else
            let p_xckrgr.not_reconciled = 0
            let p_xckrgr.adjustments = 0
            let p_xckrgr.stmt_balance = 0
            let p_xckrgr.acct_balance = 0
            let sqlca.sqlcode = 0
        end if
    else
        let p_xckrgr.not_reconciled = 0
        let p_xckrgr.adjustments = 0
        let p_xckrgr.stmt_balance = 0
        let p_xckrgr.acct_balance = 0
        let sqlca.sqlcode = 0
    end if
    whenever error call error_handler

end function
# scr_h_rec_prep()
