########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ap.4gm/i_contrl.4gs/fg_funcs.4gl  1.1  Delta: 2/4/91
########################################################################

globals "globals.4gl"

define
    nastiness_occurred  smallint

#######################################################################
function open_setup()
# deletes all a/p transactions from stxtranr, stptranr and stpactvd
#######################################################################
# this is dangerous.  if any posting is done to a/p from any journal,
# then the stxtranr record is deleted.  this will leave orphans from
# stgtranr and stgactvd and any other module transaction/activity files
# that were affected.  this is only here so the setup complete flag can
# be reset if it was set to "Y" accidentally.
#
    define
        prompt_response char(1)        # single char Y/N response

    open window opn_setup at 14,16 with 9 rows, 51 columns
        attribute (border, blue, prompt line last)
    display "===================================================" at 2,1
    display " You have indicated that accounts payable setup    " at 3,1
    display " is no longer complete.  If you wish to continue,  " at 4,1
    display " all A/P transactions will be deleted.  The open   " at 5,1
    display " items will remain.  This is generally not a good  " at 6,1
    display " idea if any A/P posting has been done.            " at 7,1
    display "===================================================" at 8,1
    let prompt_response = null
    while prompt_response matches "[^yYnN]" or
          prompt_response is null
        prompt " Continue? " for prompt_response
    end while
    if upshift(prompt_response) = "N"
    then
        close window opn_setup
        let p_pcntrc.ap_balanced = "Y"
        display by name p_pcntrc.ap_balanced
        return true
    end if

    display " Deleting A/P transactions ... please wait.        " at 1,1
    whenever error continue
    let nastiness_occurred = false
    lock table stxtranr in share mode   # lock tables for mass delete
    if sqlca.sqlcode != 0
    then
        let nastiness_occurred = true
        call fg_err(29) # Could not lock stxtranr to open setup
    end if
    if not nastiness_occurred
    then
        lock table stptranr in share mode
        if sqlca.sqlcode != 0
        then
            let nastiness_occurred = true
            call fg_err(30) # Could not lock stptranr to open setup
        end if
    end if
    if not nastiness_occurred
    then
        lock table stpactvd in share mode
        if sqlca.sqlcode != 0
        then
            let nastiness_occurred = true
            call fg_err(31) # Could not lock stpactvd to open setup
        end if
    end if
    if nastiness_occurred
    then
        display " Some nastiness has occurred ... rolling back.     "
            at 1,1
        sleep 5
        close window opn_setup
        let sqlca.sqlcode = 0
        let p_pcntrc.ap_balanced = "Y"
        display by name p_pcntrc.ap_balanced
        whenever error call error_handler
        return false
    end if
    call delt_trans()
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
    end if
    close window opn_setup

    return true

end function
# open_setup()

#######################################################################
function delt_trans()
# delete all a/p transactions
#######################################################################
#
    define
        d_journal like stptranr.orig_journal,
        d_doc_no like stptranr.doc_no,
        d_count integer

    # set control "setup completed"
    declare d_curs cursor for
        select orig_journal, doc_no from stptranr
    foreach d_curs into d_journal, d_doc_no
        select count(*) into d_count
        from stgtranr where stgtranr.orig_journal = d_journal and
            stgtranr.doc_no = d_doc_no
        if d_count = 0
        then
            delete from stxtranr
            where stxtranr.orig_journal = d_journal and
                  stxtranr.doc_no = d_doc_no
        end if
    end foreach
    close d_curs
    delete from stptranr
    delete from stpactvd

end function
# delt_trans()

#######################################################################
function close_setup()
#
#######################################################################
#
    define
        prompt_response char(1),
        tot_ap_balance decimal(16),
        pst_cd_doc_no like stpcntrc.cd_doc_no,
        pst_ap_doc_no like stpcntrc.ap_doc_no,
        hld_cd_doc_no like stpcntrc.cd_doc_no,
        hld_ap_doc_no like stpcntrc.ap_doc_no,
        hld_ap_post_no like stpcntrc.ap_post_no,
        hld_cd_post_no like stpcntrc.cd_post_no,
        oa_amt like stpvendr.on_acct_amt,
        p_popend record like stpopend.*,
        chk_amt like stpopend.balance,
        chk_disc_amt like stpopend.disc_bal,
        chk_less_disc_amt like stpopend.balance,
        vendor like stpopend.vend_code,
        c_rowid integer,
        o_rowid integer,
        x_rowid integer

    open window end_setup at 14,16 with 9 rows, 51 columns
        attribute (border, blue, prompt line last)
    display "===================================================" at 2,1
    display " You have indicated that accounts payable setup    " at 3,1
    display " is complete.  If you wish to continue, the total  " at 4,1
    display " accounts payable will be summed and displayed.    " at 5,1
    display " You will then be given the opportunity to go on   " at 6,1
    display " or to quit.                                       " at 7,1
    display "===================================================" at 8,1
    let prompt_response = null
    while prompt_response matches "[^yYnN]" or
          prompt_response is null
        prompt " Continue? " for prompt_response
    end while
    if upshift(prompt_response) = "N"
    then
        close window end_setup
        let p_pcntrc.ap_balanced = "N"
        display by name p_pcntrc.ap_balanced
        return true
    end if
    display " Summing accounts payable ... please wait.         " at 1,1
    select sum(acct_bal) into tot_ap_balance from stpvendr
    display "" at 1,2
    display " Total accounts payable is ", tot_ap_balance at 1,1
    display " At this time, you may abort this operation.  If   " at 3,1
    display " you continue, transactions will be created for    " at 4,1
    display " open items and amounts on account for vendors.    " at 5,1
    display " Once this is done, vendor statements and ledger   " at 6,1
    display " cards can be printed.                             " at 7,1
    let prompt_response = null
    while prompt_response matches "[^yYnN]" or
          prompt_response is null
        prompt "Continue? " for prompt_response
    end while
    if upshift(prompt_response) = "N"
    then
        close window end_setup
        let p_pcntrc.ap_balanced = "N"
        display by name p_pcntrc.ap_balanced
        return true
    end if
    display " Creating transaction records ... please wait.     " at 1,1

    let nastiness_occurred = false     # initialize error flag
    whenever error continue
    if not nastiness_occurred
    then
        lock table stpvendr in share mode
        if sqlca.sqlcode != 0
        then
            let nastiness_occurred = true
            call fg_err(24) # Could not lock stpvendr
        end if
    end if
    if not nastiness_occurred
    then
        lock table stpopend in share mode
        if sqlca.sqlcode != 0
        then
            let nastiness_occurred = true
            call fg_err(25) # Could not lock stpopend
        end if
    end if
    if not nastiness_occurred
    then
        lock table stxtranr in share mode
        if sqlca.sqlcode != 0
        then
            let nastiness_occurred = true
            call fg_err(26) # Could not lock stxtranr
        end if
    end if
    if not nastiness_occurred
    then
        lock table stptranr in share mode
        if sqlca.sqlcode != 0
        then
            let nastiness_occurred = true
            call fg_err(27) # Could not lock stptranr
        end if
    end if
    if not nastiness_occurred
    then
        lock table stpactvd in share mode
        if sqlca.sqlcode != 0
        then
            let nastiness_occurred = true
            call fg_err(28) # Could not lock stpactvd
        end if
    end if
    if nastiness_occurred
    then
        display " a problem occurred  ... rolling back.     "
            at 1,1
        sleep 5
        close window end_setup
        let sqlca.sqlcode = 0
        let p_pcntrc.ap_balanced = "N"
        display by name p_pcntrc.ap_balanced
        whenever error call error_handler
        return false
    end if
    let hld_cd_doc_no = m_pcntrc.cd_doc_no       # save counters
    let hld_ap_doc_no = m_pcntrc.ap_doc_no       # save counters
    let hld_ap_post_no = m_pcntrc.ap_post_no
    let hld_cd_post_no = m_pcntrc.cd_post_no
    let m_pcntrc.ap_post_no = m_pcntrc.ap_post_no + 1
    let m_pcntrc.cd_post_no = m_pcntrc.cd_post_no + 1

    call delt_trans()                   # make sure no transactions
    # set control "setup completed"
    declare c_curs cursor for
        select vend_code, on_acct_amt, rowid from stpvendr
    foreach c_curs into vendor, oa_amt, c_rowid
        let tot_ap_balance = oa_amt
        display " Working on ", vendor,
            "                                 " at 9,1

        # build on acount transaction
        if oa_amt != 0
        then
            let m_pcntrc.ap_doc_no = m_pcntrc.ap_doc_no + 1
            let pst_ap_doc_no = m_pcntrc.ap_doc_no
            insert into stxtranr values ("AP", pst_ap_doc_no,
                m_pcntrc.ap_post_no, today, today, vendor,
                "SETUP ON ACCOUNT AMOUNT")
            if sqlca.sqlcode != 0
            then
                let nastiness_occurred = true
                call fg_err(13) # Could not insert on acct into stxtranr
                exit foreach
            end if
            if oa_amt > 0
            then
                insert into stptranr 
                  values ("AP", pst_ap_doc_no, "SETUP", "CM")
            else
                insert into stptranr 
                  values ("AP", pst_ap_doc_no, "SETUP", "DM")
            end if
            if sqlca.sqlcode != 0
            then
                let nastiness_occurred = true
                call fg_err(14) # Could not insert on acct into stptranr
                exit foreach
            end if
            insert into stpactvd values ("AP", pst_ap_doc_no, "A", 0,
                oa_amt)
            if sqlca.sqlcode != 0
            then
                let nastiness_occurred = true
                call fg_err(15) # Could not insert on acct into stpactvd
                exit foreach
            end if
        end if

        # create transactions for each of the open items
        declare o_curs cursor for
            select rowid, * from stpopend
            where stpopend.vend_code = vendor
        foreach o_curs into o_rowid, p_popend.*
            let tot_ap_balance = tot_ap_balance + p_popend.balance
            # first do the invoice transactions
            select rowid into x_rowid from stxtranr
                where stxtranr.orig_journal = "AP" and
                    stxtranr.doc_no = p_popend.doc_no
            if sqlca.sqlcode = notfound
            then
                insert into stxtranr values ("AP", p_popend.doc_no,
                    m_pcntrc.ap_post_no, today, p_popend.inv_date,
                    vendor, p_popend.inv_desc)
            else
                update stxtranr set
                    (stxtranr.post_no, stxtranr.post_date) =
                    (m_pcntrc.ap_post_no, today)
                    where rowid = x_rowid
            end if
            if sqlca.sqlcode != 0
            then
                let nastiness_occurred = true
                call fg_err(16) # Could not insert invoice into stxtranr
                exit foreach
            end if
            insert into stptranr values ("AP", p_popend.doc_no,
                p_popend.inv_no, "IN")
            if sqlca.sqlcode != 0
            then
                let nastiness_occurred = true
                call fg_err(17) # Could not insert invoice into stptranr
                exit foreach
            end if
            insert into stpactvd values ("AP", p_popend.doc_no, "A",
                p_popend.doc_no, p_popend.orig_amount)
            if sqlca.sqlcode != 0
            then
                let nastiness_occurred = true
                call fg_err(18) # Could not insert invoice into stpactvd
                exit foreach
            end if

            # now do the check/discount transaction(s) if balance has changed
            let chk_amt = p_popend.balance - p_popend.orig_amount
            let chk_disc_amt = p_popend.disc_bal - p_popend.disc_amt
            if chk_amt = 0 and chk_disc_amt = 0
            then
                continue foreach
            end if
            let m_pcntrc.cd_doc_no = m_pcntrc.cd_doc_no + 1
            let pst_cd_doc_no = m_pcntrc.cd_doc_no
            insert into stxtranr values ("CD", pst_cd_doc_no,
                m_pcntrc.cd_post_no, today, p_popend.inv_date, vendor,
                "SETUP ON PAYMENT AMOUNT")
            if sqlca.sqlcode != 0
            then
                let nastiness_occurred = true
                call fg_err(19) # Could not insert payment into stxtranr
                exit foreach
            end if
            insert into stptranr values ("CD", pst_cd_doc_no, "SETUP",
                "CD")
            if sqlca.sqlcode != 0
            then
                let nastiness_occurred = true
                call fg_err(20) # Could not insert payment into stptranr
                exit foreach
            end if

            # check detail
            if chk_amt != 0
            then
                insert into stpactvd values ("CD", pst_cd_doc_no, "A",
                    p_popend.doc_no, chk_amt)
                if sqlca.sqlcode != 0
                then
                    let nastiness_occurred = true
                    call fg_err(21) # Could not insert chk into stpactvd
                    exit foreach
                end if
            end if

            # discount detail
            if chk_disc_amt != 0
            then
                insert into stpactvd values ("CD", pst_cd_doc_no, "D",
                    p_popend.doc_no, chk_disc_amt)
                if sqlca.sqlcode != 0
                then
                    let nastiness_occurred = true
                    call fg_err(22) # Could not insert dsc into stpactvd
                    exit foreach
                end if
            end if
        end foreach
        if nastiness_occurred
        then
            exit foreach
        end if
        update stpvendr set acct_bal = tot_ap_balance
            where rowid = c_rowid       # init vendor balance
        if sqlca.sqlcode != 0
        then
            let nastiness_occurred = true
            call fg_err(12) # Could not update vendor balance
            exit foreach
        end if
    end foreach
    if nastiness_occurred
    then
        display " Some nastiness has occurred ... rolling back.     "
            at 1,1
        close window end_setup
        let sqlca.sqlcode = 0
        let m_pcntrc.cd_doc_no = hld_cd_doc_no
        let m_pcntrc.ap_doc_no = hld_ap_doc_no
        let m_pcntrc.ap_post_no = hld_ap_post_no
        let m_pcntrc.cd_post_no = hld_cd_post_no
        let p_pcntrc.ap_balanced = "N"
        display by name p_pcntrc.ap_balanced
        # control "setup un-completed"
        return false
    end if
    close window end_setup
    whenever error call error_handler

    return true

end function
# close_setup()

#######################################################################
function valid_cd_acct(cash_acct_no)
#######################################################################
# return true if account number and department is defined as
# a cash account in the check register table
#
    define
        cash_acct_no like stxckrgr.acct_no,
        num_rows smallint

    select count(*) into num_rows from stxckrgr 
      where stxckrgr.acct_no = cash_acct_no

    if num_rows != 0 then return true end if

    return false

end function
# valid_cd_acct();
