######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    # Initialize some variables
    let exit_level = 0
    let tab_pressed = false


    input p_pvoidr.* without defaults from s_pvoidr.*

      # All entry fields must have before and after field processing
      before field chk_doc_no call llh_b_field("stpvoidr.chk_doc_no")
        goto next_field

      after field chk_doc_no call llh_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 1 then call llh_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after input' logic before exiting input
        call llh_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("llh_input")  # Map the key to the event
        call llh_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "chk_doc_no" next field chk_doc_no
          when nxt_fld = "exit input" goto end_input
        end case
    end input

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# llh_input()

######################################################################
function llh_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field level exit
    let exit_level = 1

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# llh_b_field()

######################################################################
function llh_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # No more exit levels required
    let exit_level = 0

    # After field logic
    case
      when scr_fld = "chk_doc_no"
        # check for change
        if dup_void_ck()
        then
            let p_pvoidr.chk_doc_no = null
            let nxt_fld = "chk_doc_no"
            return
        else
            if already_voided()
            then
                let p_pvoidr.chk_doc_no = null
                let nxt_fld = "chk_doc_no"
                return
            end if
        end if
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "chk_doc_no"
            # Perform lookups
            if llh_lookup("chk_look",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "chk_doc_no"
                return
            end if
            if llh_lookup("inv_chk_no",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "chk_doc_no"
                return
            end if
            call ap_vendn(p_pvoidr.ref_code) returning p_pvoidr.bus_name
        end case
    end if

end function
# llh_a_field()

######################################################################
function llh_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#
    if mustfill(p_pvoidr.chk_doc_no,"Check Document Number")
      then let nxt_fld = "chk_doc_no" end if

end function
# llh_a_input()

######################################################################
function llh_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# llh_event()

######################################################################
function llh_setdata()
######################################################################
# This function is called to set the this_data global variable.
#
    case
      when scr_fld = "chk_doc_no"
        call set_data(p_pvoidr.chk_doc_no)
    end case
end function
# llh_setdata()

######################################################################
function llh_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "chk_doc_no"
        display by name p_pvoidr.chk_doc_no attribute(reverse)
    end case
end function
# llh_high()

######################################################################
function llh_display()
######################################################################
# This function displays all p_* variables onto the screen
#
    display p_pvoidr.* to s_pvoidr.*  attribute(red)
end function
# llh_display()

######################################################################
function llh_p_prep()
######################################################################
# This function creates the p_* record from the m_* record
#
    let p_pvoidr.chk_doc_no = m_pvoidr.chk_doc_no

    # Perform lookups
    if llh_lookup("chk_look",false) then end if
    if llh_lookup("inv_chk_no",false) then end if

    call ap_vendn(p_pvoidr.ref_code) returning p_pvoidr.bus_name

end function
# llh_p_prep()

######################################################################
function llh_m_prep()
######################################################################
# This function creates the m_* record from the p_* record
#
    let m_pvoidr.chk_doc_no = p_pvoidr.chk_doc_no
end function
# llh_m_prep()

######################################################################
function llh_read(ring_rowid)
######################################################################
# This function loads the header p_* record elements.
# It first loads the m_* record from disk, then it
# calls the p_prep routine to fill the p_* record elements.
#
    define
        ring_rowid  integer     # rowid of record to read

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select * from stpvoidr where rowid = ?"
        prepare read_specs from scratch
        declare h_cursor cursor for read_specs
    end if

    # Read the header record
    open h_cursor using ring_rowid
    fetch h_cursor into m_pvoidr.*
    if sqlca.sqlcode != 0 then return end if
    let q_pvoidr.row_id = ring_rowid
    close h_cursor
    call llh_p_prep()
end function
# llh_read()

######################################################################
function llh_delete(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stpvoidr
        where rowid = ring_rowid
end function
# llh_delete()

######################################################################
function llh_add()
######################################################################
# This function inserts data into the header table.
#
    define
        new_rowid integer  # Rowid after insert
    # Set the serial field
    let m_pvoidr.doc_no = 0

    # Insert the data
    insert into stpvoidr values(m_pvoidr.*)
    let new_rowid = sqlca.sqlerrd[6]

    # Bring back the serial field & display it
    let m_pvoidr.doc_no = sqlca.sqlerrd[2]

    # Reset rowid
    let sqlca.sqlerrd[6] = new_rowid

end function
# llh_add()

######################################################################
function llh_update(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stpvoidr set stpvoidr.* = m_pvoidr.*
      where rowid = ring_rowid
end function
# llh_update()

######################################################################
function llh_lookup(tbl_name,mustfind)
# returning false if notfound
######################################################################
# This is the lookup function.  It is passed a table to lookup, and
# based on that, it looks up the key and fills any variables for
# that table.  The lookup function will produce the error if it is
# called with mustfind=true, and a notfound situation exists.
#
    define
        tbl_name char(18),  # Lookup table name
        stat integer,       # Value of sqlca.sqlcode after lookup
        mustfind smallint   # (boolean) produce an error if notfound?

    # Prepare cursors upon first call to this function
    if lookup_prep is null
    then
        let lookup_prep = "Y"
        let scratch = "select doc_date, doc_desc, ref_code, ",
          "post_no, post_date ",
          "from stxtranr where ",
          " stxtranr.doc_no = ? and",
          " stxtranr.orig_journal = \"CD\""
        prepare str_chk_look from scratch
        declare cur_chk_look cursor for str_chk_look
        let scratch = "select inv_chk_no, amount ",
          "from stxckrgd where ",
          " stxckrgd.doc_no = ? and",
          " stxckrgd.orig_journal = \"CD\""
        prepare str_inv_chk_no from scratch
        declare cur_inv_chk_no cursor for str_inv_chk_no
    end if

    # Perform the lookup
    case
      when tbl_name = "chk_look"
        open cur_chk_look using p_pvoidr.chk_doc_no
        fetch cur_chk_look into p_pvoidr.doc_date, p_pvoidr.doc_desc,
          p_pvoidr.ref_code, p_pvoidr.post_no, p_pvoidr.post_date
        let stat = sqlca.sqlcode
        close cur_chk_look
      when tbl_name = "inv_chk_no"
        open cur_inv_chk_no using p_pvoidr.chk_doc_no
        fetch cur_inv_chk_no into p_pvoidr.inv_chk_no, p_pvoidr.amount
        let stat = sqlca.sqlcode
        close cur_inv_chk_no
    end case

    # No mustfind if this_data is empty
    if length(this_data) = 0 then let mustfind = false end if

    # Send notfound message to user (if requested)
    let sqlca.sqlcode = stat
    if mustfind and stat = notfound
      then call scr_error("lookup=notfound",tbl_name)
    end if
    let sqlca.sqlcode = 0

    # Return false if lookup failed and empty into fields
    if stat = notfound
    then
        case
          when tbl_name = "chk_look"
            initialize p_pvoidr.doc_date, p_pvoidr.doc_desc,
              p_pvoidr.ref_code, p_pvoidr.post_no,
              p_pvoidr.post_date to null
          when tbl_name = "inv_chk_no"
            initialize p_pvoidr.inv_chk_no, p_pvoidr.amount to null
        end case
        return false
    else
        return true
    end if
end function
# llh_lookup()


#######################################################################
function already_voided()
#######################################################################
# check to see if the check has already been voided
#
    define
        counter     smallint

    select count(*) into counter from stxckrgd
      where doc_no = p_pvoidr.chk_doc_no and chk_voided = "Y"

    if counter > 0
    then
        call fg_err(2) # Check has already been voided.
        return true
    end if

    return false

end function
# already_voided()

#####################################################################
function dup_void_ck()
# checks for duplicate value in a table (used with input statements)
# returns true if a duplicate value is found, otherwise false
#####################################################################

    define
        p_rowid smallint,
        p_count smallint

    if dup_prep is null
    then
        let dup_prep = "Y"
        let scratch = "select count(*) from stpvoidr ",
                       "where chk_doc_no = ? and rowid != ?"
        prepare dup_stmt from scratch
        declare dup_cur cursor for dup_stmt
    end if

    let p_rowid = get_scrlib("curs_rowid")

    open dup_cur using p_pvoidr.chk_doc_no, p_rowid
    fetch dup_cur into p_count
    close dup_cur

    if p_count > 0
    then
        call fg_err(1)   # duplicate document
        return true
    end if
    return false

end function
# check_dup()
