########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ap.4gm/p_delete.4gs/lowlevel.4gl  1.14  Delta: 2/13/91
########################################################################

globals "globals.4gl"

define
    p_arc_date date     # archive date


#######################################################################
function before_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "first_row" then
        # insert "on first row" code here
        return
    end if

end function
# before_group()

#######################################################################
function on_every_row()
#######################################################################
#  this function prepares the report record from the cursor record
#  and other data.
#
    define
        a_key char(1)

    let rpt.co_name = curs.co_name

    # get archive date
    let p_arc_date = get_arc_date()
    call sel_wind()

    begin work

    display " Locking Tables ..." at 4,1

        # lock stxtranr, stptranr, stpactvd, stpopend, stpvendr tables
        whenever error continue
        lock table stxtranr in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(4) # Could not lock stxtranr
            rollback work
            exit program(1)
        end if
        lock table stptranr in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(5) # Could not lock stptranr
            rollback work
            exit program(1)
        end if
        lock table stpopend in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(6) # Could not lock stpopend
            rollback work
            exit program(1)
        end if
        lock table stpactvd in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(7) # Could not lock stpactvd
            rollback work
            exit program(1)
        end if
        lock table stpvendr in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(8) # Could not lock stpvendr
            rollback work
            exit program(1)
        end if
        lock table stxckrgd in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(9) # Could not lock stxckrgd
            rollback work
            exit program(1)
        end if
        whenever error call error_handler

        # delete stpactvd
        call del_actvd()

        # delete stptranr
        call del_ptranr()

        display " Removing Header Activities ..." at 13,1
        # delete stxtranr
        call del_trx(p_arc_date)

    commit work

    close window selwin
    return
end function
# on_every_row()

#######################################################################
function after_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "last_row" then
        # insert "on last row" code here
        return
    end if

end function
# after_group()

########################################################################
function del_actvd()
########################################################################
#
# delete activtiy that have zero balance and 
#   document date <= archive date
#
# delete on account activity with document date < archive date
#
    define
        vendor_code like stpvendr.vend_code, 
        p_amount like stpactvd.amount,
        p_inv_doc_no like stpactvd.inv_doc_no,
        p_rowid integer,
        quote char(1)

    let quote = "\""

    display " Removing A/P Detail Activities ..." at 7,1

    # delete all activity that does not have an assoicated open item
    declare orphan_actvd cursor for 
        select rowid from stpactvd where inv_doc_no != 0 and not exists
            (select doc_no from stpopend where doc_no = stpactvd.inv_doc_no)

    foreach orphan_actvd into p_rowid
        delete from stpactvd where rowid = p_rowid
    end foreach

    # delete all activity and open items that have 0 balance and 
    # before the given date
    declare actvd_del cursor for 
        select stpactvd.inv_doc_no 
          from stpactvd, stxtranr 
            where
              stxtranr.orig_journal = stpactvd.orig_journal and
              stxtranr.doc_no = stpactvd.doc_no and
              stpactvd.act_type = "A" and
              stpactvd.inv_doc_no != 0
            group by inv_doc_no
            having max(doc_date) <= p_arc_date and
              sum(amount) = 0)

    foreach actvd_del into p_inv_doc_no
        delete from stpopend where doc_no = p_inv_doc_no
        delete from stpactvd where inv_doc_no = p_inv_doc_no
    end foreach

    # update vendor balances from all on accounts that are to be 
    # deleted
    let ct.sel_stmt = 
       "select sum(amount), ref_code ",
         "from stxtranr, stpactvd ",
         "where ",
            "stpactvd.orig_journal = stxtranr.orig_journal and ",
            "stpactvd.doc_no = stxtranr.doc_no and ",
            "stxtranr.doc_date <= ", quote, p_arc_date, quote,
            " and stpactvd.inv_doc_no = 0 and ",
            "stpactvd.act_type = ", quote, "A", quote, 
         " group by ref_code"

    prepare sel_on_acct from ct.sel_stmt
    declare on_acct_cur cursor for sel_on_acct

    foreach on_acct_cur into p_amount, vendor_code
        update stpvendr set arch_bal = arch_bal + p_amount
            where vend_code = vendor_code
    end foreach

    display " Removing A/P On Account Activities ..." at 9,1

    # delete on account amounts
    declare on_acct cursor for 
        select stpactvd.rowid from stpactvd, stxtranr 
            where
               stpactvd.inv_doc_no = 0 and
               stpactvd.orig_journal = stxtranr.orig_journal and
               stpactvd.doc_no = stxtranr.doc_no and
               stxtranr.doc_date <= p_arc_date

    foreach on_acct into p_rowid
        delete from stpactvd where rowid = p_rowid
    end foreach

    # removed old check register detail
    declare chk_reg cursor for 
        select stxckrgd.rowid from stxckrgd, stxtranr
            where
               stxckrgd.orig_journal = stxtranr.orig_journal and
               stxckrgd.doc_no = stxtranr.doc_no and
               stxtranr.doc_date <= p_arc_date and
               stxckrgd.reconciled = "Y"

    foreach chk_reg into p_rowid
        delete from stxckrgd where rowid = p_rowid
    end foreach

end function 
# del_actvd()

########################################################################
function del_ptranr()
########################################################################
#
# delete all stptranr that do not have activity 
#
    define
        p_orig_journal like stptranr.orig_journal,
        p_doc_no like stptranr.doc_no,
        p_count integer

      display " Removing A/P Header Activities ..." at 11,1
      delete from stptranr
        where not exists
          (select rowid from stpactvd 
             where
               stpactvd.orig_journal = stptranr.orig_journal and 
               stpactvd.doc_no = stptranr.doc_no)


end function
# del_ptranr()
