########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ap.4gm/p_invpst.4gs/lowlevel.4gl  1.17  Delta: 2/13/91
########################################################################

globals "globals.4gl"

#######################################################################
function before_group(group_key)
#######################################################################
#
    define
        file_type char(2),      # IN/CM/DM
        inv_doc_no integer,     # invoice document number
        group_key   char(20),   # group identification
        old_key like stxaddld.record_key, 
        new_key like stxaddld.record_key

    # Trap fatal errors
    whenever error call error_handler

    if group_key = "first_row" then
        if check_post = "POST"
        then
            begin work
                call auto_next("stpcntrc","ap_post_no")
                    returning post_no
            commit work
            let rpt.post_seq = post_no using "Posting Sequence: <<<<<<"
        else
            let rpt.post_seq = ""
        end if
        return
    end if

    if group_key = "doc_no" then
    # For each document do the following:
    # - read stpinvce.* into p_pinvce (wouldn't all fit into cursor)
    # - begin work & set some accumulators to 0
    # - if post, then grab the next doc_no, if not, set doc_no to ""
    # - call ap_post
    # - set the appropriate rpt.* variables
    
        let sqlca.sqlcode = 0
        let rpt.problm0 = ""
        let new_doc_no = curs.doc_no
        select * into p_pinvce.* from stpinvce where doc_no = curs.doc_no
        if sqlca.sqlcode != 0
        then
            if check_post = "POST"
            then
                call fg_err(1) # Cannot read stpinvce table
            end if
            call end_report("An SQL error has occured while reading stpinvce.")
        end if
        if check_post = "POST"
        then
            begin work
            if curs.ok_to_post != "Y"
            then
                rollback work
                call fg_err(4) # edit list hasn't been run on this document
                call end_report("The edit list hasn't been run on this document.")
            end if
            call auto_next("stpcntrc","ap_doc_no")
                returning new_doc_no
            let rpt.txt_doc_no = new_doc_no using "<<<<<<<<<"
            let old_key = curs.doc_no using "<<<<<<<<<<"
            let new_key = new_doc_no using "<<<<<<<<<<"
            call xfer_udf("stpinvce","stxtranrAP",old_key,new_key,"M")
        else
            let rpt.txt_doc_no = "UNASSIGNED"
        end if
        let rpt.doc_no = curs.doc_no
        if p_pinvce.ap_debit_credit = "CR"
        then
            let ap_signed_amt = p_pinvce.ap_amount
            let disc_signed_amt = p_pinvce.disc_amount
        else
            let ap_signed_amt = p_pinvce.ap_amount * -1
            let disc_signed_amt = p_pinvce.disc_amount * -1
        end if

        if curs.file_type = "I"
        then
            let rpt.inv_type = "INVOICE"
            let rpt.ref_code = ""
            let rpt.ref_verb = ""
            let file_type = "IN"
            let inv_doc_no = new_doc_no
        else
            if curs.file_type = "C"
            then
                let rpt.inv_type = "CREDIT MEMO"
                let file_type = "CM"
            else
                let rpt.inv_type = "DEBIT MEMO"
                let file_type = "DM"
            end if
            let rpt.ref_verb = "Reference:"
            if curs.ref_no is null
            then
                let rpt.ref_code = "ON ACCOUNT"
                let inv_doc_no = 0
            else
                let rpt.ref_code = curs.ref_no using "<<<<<<<<<<"
                let inv_doc_no = curs.ref_no
            end if
        end if # invoice / memo

        # post this guy to a/p...
        if ap_post(
            check_post, "AP", new_doc_no, post_no, today, p_pinvce.doc_date,
            p_pinvce.vend_code, p_pinvce.inv_desc, p_pinvce.inv_no,
            file_type, "A", inv_doc_no, ap_signed_amt, p_pinvce.disc_date,
            disc_signed_amt,
            ap_signed_amt, disc_signed_amt, p_pinvce.to_pay_date,
            p_pinvce.pay_to_code,
            p_pinvce.due_date, p_pinvce.ap_acct_no, p_pinvce.ap_department, 
            p_pinvce.po_no, p_pinvce.po_date,p_pinvce.inv_date,
            p_pinvce.cash_acct_no, p_pinvce.cash_department) = false
        then
            if check_post = "POST"
            then
                rollback work
                call fg_error("lib_all","ap_post",post_ap.status)
            end if
            call end_report(post_ap.description)
        end if

        # cause fatal error if a/p isn't installed (very unlikely)
        if post_ap.status = 1
        then
            rollback work
            call fg_error("lib_all","ap_post",post_ap.status)
            let check_post = "POST"  # make end_report fatal...
            call end_report(post_ap.description)
        end if

        # no more updating necessary if edit list
        if check_post = "CHECK"
        then
            update stpinvce set ok_to_post = "Y"
                where doc_no = curs.doc_no
            if sqlca.sqlcode != 0
            then
                call fg_err(5) # cannot update stpinvce
                call end_report("cannot update stpinvce")
            end if
            return
        end if

        return
    end if

end function
# before_group()

#######################################################################
function on_every_row()
#######################################################################
#  this function prepares the report record from the cursor record
#  and other data.
#
    let rpt.file_type = curs.file_type
    let rpt.debit_credit = curs.debit_credit
    let rpt.line_no = curs.line_no
    let rpt.due_date = curs.due_date
    let rpt.disc_date = curs.disc_date
    let rpt.po_no = curs.po_no
    let rpt.po_date = curs.po_date
    let rpt.doc_date = curs.doc_date
    let rpt.vend_code = curs.vend_code
    let rpt.bus_name = ap_vendn(rpt.vend_code)
    let rpt.inv_no = curs.inv_no
    let rpt.inv_desc = curs.inv_desc
    let rpt.inv_date = curs.inv_date
    let rpt.acct_no = curs.acct_no
    let rpt.department = curs.department
    let rpt.amount = curs.amount
    let rpt.debit_credit = curs.debit_credit

    # post to g/l...
    if gl_post(
        check_post, "AP", new_doc_no, post_no, today, p_pinvce.doc_date,
        p_pinvce.vend_code, p_pinvce.inv_desc, p_pinvce.inv_no, curs.acct_no,
        curs.department, curs.amount, curs.debit_credit
    ) = false
    then
        if check_post = "POST"
        then
            rollback work
            call fg_error("lib_all","gl_post",post_gl.status)
        end if
        call end_report(post_gl.description)
    end if

    # account name is stored in post_gl.acct_desc
    let rpt.description = post_gl.acct_desc

    return
end function
# on_every_row()

#######################################################################
function after_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "doc_no" then
        call last_few() # print the last few distributions
        if gl_last() = false
        then
            if check_post = "POST"
            then
                rollback work
                call fg_error("lib_all","gl_last",post_gl.status)
            end if
            call end_report(post_gl.description)
        end if

        if ap_last(check_post, "IN", ap_signed_amt, curs.vend_code) = false
        then
            if check_post = "POST"
            then
                rollback work
                call fg_error("lib_all","ap_last",post_ap.status)
            end if
            call end_report(post_ap.description)
        end if

        # either delete the entry, or mark it as posted (only if recurring)
        if check_post = "POST"
        then
          # - set posted to "Y" in stpinvce (if not deleted)
          if p_pinvce.recurring is not null
          then
            update stpinvce set (posted, recurr_ref) = ("Y", rpt.txt_doc_no)
                where doc_no = curs.doc_no
            if sqlca.sqlcode != 0
            then
              rollback work
              call fg_err(2) #An SQL error has occured while posting to stpinvce
              call end_report(
                "An SQL error has occured while posting the previous document.")
            end if
          else
            # - delete stpinvce unless it's marked for recurring
            delete from stpinvce 
                where doc_no = curs.doc_no
            if sqlca.sqlcode = 0
            then
              delete from stpinvcd 
                  where doc_no = curs.doc_no
            end if
            if sqlca.sqlcode != 0
            then
              rollback work
              call fg_err(3) #An SQL error has occured while deleting stpinvce
              call end_report(
              "An SQL error has occured while deleting the previous document.")
            end if
          end if
        end if

        if check_post = "POST"
        then
            commit work
        end if

        return
    end if

    if group_key = "line_no" then
        # insert "after group of line_no" code here
        return
    end if

    if group_key = "last_row" then
        if rpt.problm1 is null
        then
            let rpt.problm1 =
                "**** End of Report."
        end if
        return
    end if

end function
# after_group()

#######################################################################
function last_few()
#######################################################################
#  this function posts the last few lines of the invoice to distd
#  and to the report.
#
    if curs.line_no is null
    then
        let curs.line_no = 0
    end if

    if p_pinvce.ap_amount != 0
    then
        call ml_output()       # output the current row
        let curs.line_no = curs.line_no + 1
        let curs.acct_no = p_pinvce.ap_acct_no
        let curs.department = p_pinvce.ap_department
        let curs.amount = p_pinvce.ap_amount
        let curs.debit_credit = p_pinvce.ap_debit_credit
        call on_every_row()    # prepare new row data
    end if
    
end function
# last_few()

#######################################################################
function end_report(why)
#######################################################################
#  this function calls the last line of the report for abnormal termination.
#
    define
        why char(80)

    if check_post = "POST"
    then
        let rpt.problm1 =
            "***** The posting has terminated for the following reason:"
        let rpt.problm2 = "***** ", why clipped
        let rpt.problm3 =
            "***** It HAS NOT been posted.  Documents prior to it HAVE been posted."
        call ml_output()
        finish report report1
        exit program(1)
    else
        let rpt.problm0 = "***** ",why clipped
        let rpt.problm1 = "***** ERRORS HAVE BEEN DETECTED ON THIS REPORT"
    end if

end function
# end_report()
